"""
Contains some fields as utilities
"""
from django import forms
from django.conf import settings
from django.utils.safestring import mark_safe

class SubFKWidget(forms.Select):
    """
    This widget creates a formfield for a foreignkey but linked to its subclasses.
    sub_options parameter is a list of options for every subclass linked to the field with this widget.
    Every item of this list has these following parameters:
        - application: application's name
        - model: model's name
        - html: represents the html that you want insert in the popup's link
    refer_id: represents css id of object needs add button
    calling: represents object that calls createAddButton
    """
    def __init__(self,sub_options = [],refer_id = '',calling = '', *args, **kwargs):
        """
        Takes following parameters:
        sub_options: is a list of dictionaries. Each dictionary has application (lower application's name), model (lower model's name), html (html for add popup link),
        refer_id: represents css id of object needs add button,
        calling: represents object that calls createAddButton
        """
        self.sub_options = sub_options
        self.refer_id = refer_id
        self.calling = calling
        self.validate_sub_options()
        super(SubFKWidget, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None):
        """
        Renders this widget
        """
        rendered = super(SubFKWidget, self).render(name, value, attrs)
        return rendered + mark_safe(u"%s" % self.createAddButton())

    def createAddButton(self):
        """
        Creates html to add to widget
        """
        buttons = u""
        print "\n\n",self.__dict__
        for item in self.sub_options:
            application = item['application']
            model = item['model']
            html = item['html']
            buttons += '<span class = "add_custom_button"><a href="/admin/'+application+'/'+model+'/add/" class="add-another" id="add_'+self.refer_id+'" onclick="return showAddAnotherPopupCustom(this,\''+self.calling+'\');">'+html+'</a></span>';
        return buttons
    
    def validate_sub_options(self):
        """
        Validates sub_options parameter
        """
        if not self.sub_options or self.sub_options == []:
            raise ValueError("Error in upy.widgets.SubFKWidget: sub_options can't be empty or none. Check configurations in model")
        for item in self.sub_options:
            if not item.has_key('application'):
                raise ValueError("Error in upy.widgets.SubFKWidget: missing argument 'application' in sub_options")
            if not item.has_key('model'):
                raise ValueError("Error in upy.widgets.SubFKWidget: missing argument 'model' in sub_options")
            if not item.has_key('html'):
                raise ValueError("Error in upy.widgets.SubFKWidget: missing argument 'html' in sub_options")
    
    class Media:
        js = (settings.JQUERY_LIB,
              '/upy_static/js/sub_foreignkey.js',)
        css = {"all" : ("/upy_static/css/sub_foreignkey.css",)}

    


from django.template import Library
from upy.contrib.customadmin.models import CustomApp,CustomModel

register = Library()

@register.filter
def add_app_icons(app_list, autocomplete):
    """
    It returns a list of applications to view in admin's interface (index.html). 
    This filter is called only if you want view icons mode.
    """
    app_icon_list = CustomApp.objects.all()
    app_list_ok = []
    for appicon in app_icon_list:
        try:
            app_temp = [x for x in app_list if x.get('name').lower() == appicon.application.lower()][0]
            app_temp['image'] = appicon.image
            app_temp['verbose_app_name'] = appicon.verbose_app_name
            app_list_ok.append(app_temp)
            #app_list_ok.extend([x for x in app_list if x.get('name') == appicon.application])
        except IndexError, e:
            print ("Warning! In customadmin.templatetags.add_app_icons: ", 
                   e, "; application: ", appicon.application)
            pass # probabilmente e' stata inserita nella lista di applicazioni con icona 
                # una applicazione che non e' tra quelle registrate nell'admin, 
                # quindi poco male...
      
    if autocomplete:  
        app_list_ok_name = [x.get('name') for x in app_list_ok]
        app_list_name = [x.get('name') for x in app_list]
        app_out_name = list(set(app_list_name)-set(app_list_ok_name)) 
        app_out_list = [x for x in app_list if x.get('name') in app_out_name]
        
        for app_out in app_out_list:
            app_out['image'] = None
            app_out['verbose_app_name'] = app_out.get('name')
        
        app_list_ok.extend(app_out_list)
        
    return app_list_ok


@register.filter
def custom_app_list(app_list, autocomplete):
    """
    It returns a list of applications chosen to view in admin's interface (index.html).
    """
    app_icon_list = CustomApp.objects.all()
    app_list_ok = []
    
    for appicon in app_icon_list:
        try:
            app_temp = [x for x in app_list if x.get('name').lower() == appicon.application.lower()][0]
            app_temp['show_models'] = appicon.show_models
            app_temp['verbose_app_name'] = appicon.verbose_app_name
            app_list_ok.append(app_temp)
            #app_list_ok.extend([x for x in app_list if x.get('name') == appicon.application])
        except IndexError, e:
            print ("Error in customadmin.templatetags.add_app_icons: ", e, 
                   "; application: ", appicon.application)
            pass # probabilmente e' stata inserita nella lista di applicazioni con icona una applicazione
            # che non e' tra quelle registrate nell'admin, quindi poco male...
    
    if autocomplete:
        app_list_ok_name = [x.get('name') for x in app_list_ok]
        app_list_name = [x.get('name') for x in app_list]
        app_out_name = list(set(app_list_name)-set(app_list_ok_name)) 
        app_out_list = [x for x in app_list if x.get('name') in app_out_name]
        
        for app_out in app_out_list:
            app_out['show_models'] = True
            app_out['verbose_app_name'] = app_out.get('name')
    
        app_list_ok.extend(app_out_list)
       
    return app_list_ok



@register.filter
def add_model_icons(models_list,autocomplete):
    """
    It returns a list of models to view in admin's interface (index.html). 
    This filter is called only if you want view icons mode for models.
    """
    models_icon_list = CustomModel.objects.all()
    models_list_ok = []
    
    for cmodel in models_icon_list:
        try:
            app_temp_l = []
            for x in models_list:
                if u"%s" % x.get('name').lower() == u"%s" % cmodel._meta.verbose_name_plural.lower():
                    app_temp_l.append(x)
            app_temp = app_temp_l[0]
            
            app_temp['image'] = cmodel.image
            models_list_ok.append(app_temp)
        except IndexError, e:
            print ("Warning! In customadmin.templatetags.add_model_icons: ", 
                   e, "; application: ", cmodel.model)
            pass # probabilmente e' stato inserito nella lista di models con icona 
                # un model che non e' tra quelli registrate nell'admin, 
                # quindi poco male...
    
    if autocomplete:  
        models_list_ok_name = [x.get('name') for x in models_list_ok]
        models_list_name = [x.get('name') for x in models_list]
        models_out_name = list(set(models_list_name)-set(models_list_ok_name)) 
        models_out_list = [x for x in models_list if x.get('name') in models_out_name]
        
        for model_out in models_out_list:
            model_out['image'] = None
            
        models_list_ok.extend(models_out_list)   
    return models_list_ok


from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string
from email.MIMEImage import MIMEImage
import mimetypes

def send_mail(subject, text_content, from_email, to, html_content = None, attachments = None, publication = ""):
    """
    This function sends mail using EmailMultiAlternatives and attachs all attachments
    passed as parameters
    """
    msg = EmailMultiAlternatives(subject, text_content, from_email, to)
    if html_content:
        msg.attach_alternative(html_content, "text/html")
    if attachments:
        for att in attachments:            
            mimetype, encoding = mimetypes.guess_type(att.file.url)
            if str(mimetype) in ('image/jpeg', 'image/pjpeg', 'image/png', 'image/gif'):
                try:
                    with open(att.file.path) as f:
                        email_embed_image(msg, att.name, f.read())
                except Exception, e:
                    print e
            else:
                msg.attach_file("%s" % (att.file.url[1:]))
    try:
        msg.send()
    except Exception, e:
        print e
        
    
def send_rendered_mail(subject, text_content, template_name, context_dict, from_email,to, attachments = None):
    """
    It sends mail after rendering html content
    """
    try:
        rendered = render_to_string(template_name,context_dict)
    except Exception, e:
        print e
    return send_mail(subject, text_content, from_email, to, rendered, attachments)


def email_embed_image(email, img_content_id, img_data):
    """
    email is a django.core.mail.EmailMessage object
    """
    img = MIMEImage(img_data)
    img.add_header('Content-ID', '<%s>' % img_content_id)
    img.add_header('Content-Disposition', 'inline')
    email.attach(img)
    


from django.utils.translation import ugettext_lazy as _
from django.db import models
from imagekit.models import ImageSpecField, ProcessedImageField
from imagekit.processors import ResizeToFit, SmartResize, Adjust
from upy.utils import today
from django.conf import settings
from project import config
from upy.contrib.g11n.models import G11nModel

def _choose_preprocessor():
    """
    Function to define the preprocessor on original_image field. It crops the original image, so user
    cannot save on server too big images. (in order to prevent disk space wasting)
    """
    if settings.USE_FULLHD_SUPPORT:
        return ResizeToFit(width=1920, height=1366) #upscale=False)
    else:
        return ResizeToFit(width=800, height=800) #upscale=False)

    
class UPYImage(models.Model):
    """
    This abstract class only provide original_image and admin_thumbnail field, with the 
    admin_thumbnail_view function. Inherit this class in you models.py and import imagekit ImageSpec
    to create other thumbnail or display field with any processor you need.
    """
    original_image = ProcessedImageField(null = True, blank = True, upload_to='upyimage', processors=[_choose_preprocessor(),])
    admin_thumbnail = ImageSpecField([Adjust(contrast=1.2, sharpness=1.1),
            SmartResize(25, 25)], image_field='original_image', options={'quality': 90})  #format='JPEG', 

    def admin_thumbnail_view(self):
        """
        It's a property used in admin interface. It decorates list_display items with a thumbnail 
        and with prettyPhoto jQuery plugin to open images in a lightbox.
        """
        if self.original_image:
            return u'<a href="%s" data-rel="prettyPhoto" title="%s"><img src="%s" class="upy_admin_thumb" alt="%s"/></a>' % (self.original_image.url, _("view image"), self.admin_thumbnail.url, self.admin_thumbnail.url )
        else:
            return None
    admin_thumbnail_view.short_description = _(u'Thumbnail')
    admin_thumbnail_view.allow_tags = True
    
    original_image_label = _(u"Original image")
    
    def __unicode__(self):
        return u"%s" % (self.original_image)

    class Meta:
        abstract = True
                
class PositionImage(UPYImage):
    """
    Class for Images with position. It adds position field and manages position
    on admin interface.
    """
    position = models.PositiveSmallIntegerField(u'Position', default=0)

    class Meta: 
        ordering = [u'position',]
        abstract = True   
            
            
if config.USE_UPY_G11N:
        
    class UPYImageG11n(G11nModel):
        """
        Abstract class for G11nModel to inherit if you need localized contents for your UPYImage model
        """
        title = models.CharField(max_length = 150, help_text = _(u"Set the image title."))
        alt = models.CharField(max_length = 150, help_text = _(u"Set the image alternative field."))
        
        def __unicode__(self):
            return u"%s" % (self.language)
        
        class Meta:
            abstract = True

if config.USE_UPY_IMAGE:            
    class ImageRepositoryBase(PositionImage):
        """
        Concrete class for Images. It add useful fields like position, name, date... and manage position
        on the admin.
        """
        name = models.CharField(max_length=100)
        display_image = ImageSpecField([ResizeToFit(800,600)], image_field='original_image', options={'quality': 90})  #format='JPEG',
        date = models.DateField(default = today(), help_text = _(u"Set the date you want to display for this image.")) 
        creation_date = models.DateTimeField(auto_now_add = True, help_text = _(u"Establishment date"), 
                                             verbose_name = _(u"Creation date"))
        def admin_thumbnail_view(self):
            """
            It's a property used in admin interface. It decorates list_display items with a thumbnail 
            and with prettyPhoto jQuery plugin to open images in a lightbox.
            """
            if self.original_image:
                return u'<a href="%s" data-rel="prettyPhoto" title="%s"><img src="%s" alt="%s"/></a>' % (self.display_image.url, _("view image"), self.admin_thumbnail.url, self.admin_thumbnail.url )
            else:
                return None
        admin_thumbnail_view.short_description = _(u'Thumbnail Display')
        admin_thumbnail_view.allow_tags = True
        
        def get_name(self):
            return self.name
        
        def __unicode__(self):
            return u"%s" % (self.name)
            
        class Meta:
            verbose_name = _(u"Image")
            verbose_name_plural = _(u"Images")
            ordering = [u'position',]
            abstract = True
    
        
    if config.USE_UPY_G11N:
        
                
        class ImageRepositoryG11n(UPYImageG11n):
            """
            It defines image's repository g11n model if G11n is used in this project
            """
            description = models.TextField(null = True, blank = True, help_text = _(u"Image description."))
            image_repository = models.ForeignKey(u"ImageRepository", help_text = _(u"Choose an Image Repository element to associate."), verbose_name = _(u"Image Repository element"))
    
        class ImageRepository(ImageRepositoryBase):
            """
            It defines image's repository model if G11n is used in this project
            """
            class G11nMeta:
                g11n = 'ImageRepositoryG11n'
                fieldname = 'image_repository'
    
    else:
        
        class ImageRepository(ImageRepositoryBase):
            """
            It defines image's repository model if G11n is not used in this project
            """
            title = models.CharField(max_length = 150, help_text = _(u"Set the image title."))
            alt = models.CharField(max_length = 150, help_text = _(u"Set the image alternative field."))
            description = models.TextField(null = True, blank = True, help_text = _(u"Image description."))